@@ -1,3 +1,4 @@
+from warnings import warn
 from s3fs import S3FileSystem
 from pandas import read_csv
 from pandas import DataFrame as PandasDF
@@ -5,7 +6,6 @@ from pickle import dump as pickle_dump
 from pickle import load as pickle_load
 from psycopg2 import connect as psycopg2_connect
 from csv import QUOTE_NONNUMERIC
-from pyspark.sql.session import SparkSession
 from pyspark.sql.dataframe import DataFrame as SparkDF
 from .S3File import S3Files
 
@@ -18,7 +18,7 @@ class S3:
 		:type secret: str or NoneType
 		:type iam_role: str or NoneType
 		:type root: str or NoneType
-		:type spark: SparkSession or NoneType
+		:type spark: pyspark.sql.session.SparkSession or NoneType
 		"""
 		self._key = key
 		self._secret = secret
@@ -28,11 +28,9 @@ class S3:
 		self._root = root
 		self._spark = spark
 
-		self.list = self.ls
-		self.move = self.mv
-		self.dir = self.ls
-		self.copy = self.cp
-		self.delete = self.rm
+	@property
+	def root(self):
+		return self._root
 
 	@property
 	def file_system(self):
@@ -219,12 +217,18 @@ class S3:
 	@property
 	def spark(self):
 		"""
-		:rtype: SparkSession
+		:rtype: pyspark.sql.session.SparkSession
 		"""
 		if self._spark is None:
 			return RuntimeError('S3 does not have access to a SparkSession')
 		return self._spark
 
+	def set_spark(self, spark):
+		"""
+		:type spark: pyspark.sql.session.SparkSession
+		"""
+		self._spark = spark
+
 	@staticmethod
 	def get_file_name_and_extension(path):
 		"""
@@ -282,7 +286,7 @@ class S3:
 		"""
 		reads parquet files inside a path and returns the data
 		:type path: str
-		:type spark: SparkSession or NoneType
+		:type spark: pyspark.sql.session.SparkSession or NoneType
 		:type parallel: bool
 		:rtype: SparkDF
 		"""
@@ -314,6 +318,11 @@ class S3:
 
 			return result
 
+	list = ls
+	move = mv
+	dir = ls
+	copy = cp
+	delete = rm
 	load_parquet = read_parquet
 	save_parquet = write_parquet
 	load_pickle = read_pickle
@@ -334,7 +343,7 @@ class S3:
 			if not path.endswith('.parquet'):
 				path += '.parquet'
 
-			return self.write_parquet(data=obj, path=path, mode=mode)
+			self.write_parquet(data=obj, path=path, mode=mode)
 
 		else:
 			if path.endswith('.parquet'):
@@ -343,14 +352,16 @@ class S3:
 			if not path.endswith('.pickle'):
 				path += '.pickle'
 
-			return self.write_pickle(obj=obj, path=path, mode=mode)
+			self.write_pickle(obj=obj, path=path, mode=mode)
+
+		return self / path
 
 	def load(self, path, spark=None):
 		"""
 		reads an object from a path. If the extension is .parquet then the object is assumed to be a Spark DataFrame
 		and the read_parquet method is used, otherwise a pickle will be read.
 		:type path: str
-		:type spark: SparkSession
+		:type spark: pyspark.sql.session.SparkSession
 		:rtype: bool
 		"""
 		if path.endswith('.parquet'):
@@ -361,3 +372,96 @@ class S3:
 
 		else:
 			raise NotImplementedError(f'load is not implemented for {path}!')
+
+	def __truediv__(self, other):
+		"""
+		:type other: str
+		:rtype: S3Path
+		"""
+		if not isinstance(other, str):
+			warn(f'other is of type "{type(other)}" and is cast as string!')
+			other = str(other)
+		return S3Path(s3=self, path=other)
+
+
+class S3Path:
+	def __init__(self, s3, path):
+		"""
+		:type s3: S3
+		:type path: str
+		"""
+		if path.startswith(s3.root):
+			warn(f'path "{path}" includes S3 root "{s3.root}"!')
+			path = path[len(s3.root):]
+
+		self._s3 = s3
+		self._path = path
+
+	def __truediv__(self, other):
+		"""
+		:type other: str
+		:rtype: S3Path
+		"""
+		if not isinstance(other, str):
+			warn(f'other is of type "{type(other)}" and is cast as string!')
+			other = str(other)
+		left = self._path.rstrip('/')
+		right = other.lstrip('/')
+		return self.__class__(s3=self.s3, path=f'{left}/{right}')
+
+	__add__ = __truediv__
+
+	def __repr__(self):
+		path = self._path
+		if path.lower().startswith('s3'):
+			return path
+		else:
+			return f's3://{path}'
+
+	@property
+	def s3(self):
+		"""
+		:rtype: S3
+		"""
+		return self._s3
+
+	@property
+	def spark(self):
+		"""
+		:rtype: pyspark.sql.session.SparkSession
+		"""
+		return self.s3.spark
+
+	def set_spark(self, spark):
+		"""
+		:type spark: pyspark.sql.session.SparkSession
+		"""
+		self.s3.set_spark(spark)
+
+	def load(self, spark=None):
+		return self.s3.load(path=self._path, spark=spark)
+
+	def save(self, obj, mode='overwrite'):
+		return self.s3.save(obj=obj, path=self._path, mode=mode)
+
+	def ls(self, **kwargs):
+		return self.s3.ls(path=self._path, **kwargs)
+
+	def is_file(self):
+		return self.s3.is_file(path=self._path)
+
+	def is_dir(self):
+		return self.s3.is_dir(path=self._path)
+
+	@property
+	def size(self):
+		return self.s3.get_size(path=self._path)
+
+	def exists(self):
+		return self.s3.exists(path=self._path)
+
+	def delete(self):
+		return self.s3.delete(path=self._path)
+
+	dir = ls
+	list = ls
